#!/usr/bin/env python2

import pygame, sys, random
from pygame.locals import *

SCREENWIDTH = 1024
SCREENHEIGHT = 768
pygame.init()
#SCREEN = pygame.display.set_mode((SCREENWIDTH, SCREENHEIGHT), pygame.FULLSCREEN)
SCREEN = pygame.display.set_mode((SCREENWIDTH, SCREENHEIGHT), 0)
FONT = pygame.font.Font('freesansbold.ttf', 16)
FPSCLOCK = pygame.time.Clock()
FPS = 24
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
YELLOW = (255, 255, 0)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
class Gamer (object):
	def __init__ (self):
		self.score = 0
		self.lives = 5

def menuText (pos, c1, c2, text):
	txtsurf = FONT.render(text, True, c1, c2)
	txtrect = txtsurf.get_rect()
	txtrect.topleft = ((pos[0], pos[1]))
	txt = SCREEN.blit(txtsurf, txtrect)
	return txt

def titleScreen ():
	done = False
	redo = True
	updates = []
	while not done:
		if redo:
			SCREEN.blit (pygame.image.load('background_title.png').convert(), (0, 0))
			pygame.display.update ()
			redo = False

		startButton = menuText ((0, 0), BLACK, WHITE, "Start")
		updates.append (startButton)
		infoButton = menuText ((0, 50), BLACK, WHITE, "Info")
		updates.append (infoButton)
		quitButton = menuText ((0, 100), BLACK, WHITE, "Quit")
		updates.append (quitButton)

		for event in pygame.event.get():
			if event.type == QUIT:
				sys.exit()
			elif event.type == MOUSEBUTTONDOWN:
				if startButton.collidepoint (event.pos):
					doGame ()
					redo = True
				elif infoButton.collidepoint (event.pos):
					staticScreen ('info')
					redo = True
				elif quitButton.collidepoint (event.pos):
					done = True
            			
		pygame.display.update (updates)
		updates = []
		FPSCLOCK.tick (FPS)

def staticScreen (screen):
	done = False
	updates = []
	SCREEN.blit (pygame.image.load('background_' + screen + '.png').convert(), (0, 0))
	pygame.display.update ()
	while not done:
		okButton = menuText ((0, 0), BLACK, WHITE, "OK")
		updates.append (okButton)

		for event in pygame.event.get():
			if event.type == QUIT:
				sys.exit()
			elif event.type == MOUSEBUTTONDOWN:
				if okButton.collidepoint (event.pos):
					done = True
            			
		pygame.display.update (updates)
		updates = []
		FPSCLOCK.tick (FPS)

class Pad (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y

class Star (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(BLACK)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y
	        
	def update (self):
		if random.randrange (10000) >= 9999:
			for g in self.groups:
				self.remove (g)
				
class Lander (pygame.sprite.DirtySprite):
	def __init__(self, x, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = 0
		self.fuel = 100 
		self.thrust_left = False
		self.thrust_right = False
		self.thrust_up = False
		self.down = 0.0
		self.left = 0
		self.right = 0
	        
	def update (self):
		# Should check if we are too far left or right...
		if self.fuel > 0:
			if self.thrust_left:
				self.fuel = self.fuel - 1
				self.left = self.left + 1
			if self.thrust_right:
				self.fuel = self.fuel - 1
				self.right = self.right + 1
			if self.thrust_up:
				self.fuel = self.fuel - 1
				self.down = self.down - 0.5
				# should check if we've flown off into space...
		else:
			self.thrust_left = False
			self.thrust_right = False
			self.thrust_up = False
			
		if not self.thrust_up:
			self.down = self.down + 0.5
			
		horizontal = self.left - self.right
		self.rect.left = self.rect.left + horizontal
		
		vertical = self.down
		self.rect.top = self.rect.top + vertical
            
def game1 (gamer):
	redo = True
	updates = []
	lander_img = pygame.image.load('lander.png').convert()
	pad_img = pygame.image.load('pad.png').convert()
	star_img = pygame.image.load('star.png').convert()
	done = False	
	
	while not done:
		if redo == True:
			background_img = pygame.image.load('background_game1.png').convert()
			SCREEN.blit (background_img, (0, 0))
			pygame.display.update ()
			star_sprites = pygame.sprite.RenderUpdates ()
			player_sprites = pygame.sprite.RenderUpdates ()
			all_sprites = pygame.sprite.RenderUpdates ()
			
			player = Lander (random.randrange(SCREENWIDTH), lander_img, [player_sprites, all_sprites])
			player_sprites.add (player)		
			all_sprites.add (player)		
			
			pad = Pad (320, 700, pad_img, [all_sprites])
			all_sprites.add (pad)			
			redo = False
			
			go = False
		
		while len (star_sprites) < 50:
			star = Star (random.randrange(SCREENWIDTH), random.randrange(SCREENHEIGHT), star_img, [star_sprites, all_sprites])
			star_sprites.add (star)
			all_sprites.add (star)		
		
		if player.rect.bottom == pad.rect.top:
			if player.rect.left >= pad.rect.left:
				if player.rect.right <= pad.rect.right:
					if player.down > 10:
						gamer.lives = gamer.lives - 1
						staticScreen ('died')
						redo = True
					else:
						done = True
						gamer.score = gamer.score + (20 - (pad.rect.left - player.rect.left)) # This is wrong
		elif player.rect.bottom > pad.rect.top:
			print 'missed'
			gamer.lives = gamer.lives - 1
			staticScreen ('died')
			redo = True

		for event in pygame.event.get():
			if event.type == QUIT:
				sys.exit()
		keys = pygame.key.get_pressed()  #checking pressed keys
		if keys[pygame.K_RIGHT]:
			player.thrust_left = True
		else:
			player.thrust_left = False
		if keys[pygame.K_LEFT]:
			player.thrust_right = True
		else:
			player.thrust_right = False
		if keys[pygame.K_UP]:
			player.thrust_up = True
		else:
			player.thrust_up = False
		if keys[pygame.K_DOWN]:
			go = True

		all_sprites.clear (SCREEN, background_img)

		star_sprites.update ()
		
		if go:
			player_sprites.update ()

		updates.extend (all_sprites.draw (SCREEN))
		
		updates.append (pygame.draw.rect (SCREEN, BLACK, Rect (0, 0, SCREENWIDTH, 15)))
		menuText ((0, 0), WHITE, BLACK, "Fuel " + str (player.fuel) + "%")
		if player.down <7:
			c = GREEN
		elif player.down > 10:
			c = RED
		else:
			c = YELLOW
		menuText ((90, 0), WHITE, BLACK, "Lives " + str (gamer.lives))
		menuText ((180, 0), WHITE, BLACK, "Score " + str (gamer.score))
		menuText ((270, 0), c, BLACK, "Speed " + str (player.down))
		updates.append (menuText ((0, 20), WHITE, BLACK, "Land your lunar module using the arrow keys. Don't run out of fuel, don't hit the ground too fast. Tap down when you are ready."))
			
		
		pygame.display.update (updates)
		updates = []
		FPSCLOCK.tick (FPS)
	return gamer

class Miner (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y
		self.direction = 'stop'
		self.speed = 5
		self.weapon = pygame.time.get_ticks()
		self.helium = 0
	def update (self, heliums, weapons):
		if self.direction == 'up':
			self.rect.y = self.rect.y - self.speed
		elif self.direction == 'down':
			self.rect.y = self.rect.y + self.speed
		elif self.direction == 'left':
			self.rect.x = self.rect.x - self.speed
		elif self.direction == 'right':
			self.rect.x = self.rect.x + self.speed
		
		
class Rock (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y

class BadMiner (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y
		self.direction = 'stop'
		self.speed = 5
	def update (self, rocks):
		ox = self.rect.x
		oy = self.rect.y
		if self.direction == 'up':
			self.rect.y = self.rect.y - self.speed
		elif self.direction == 'down':
			self.rect.y = self.rect.y + self.speed
		elif self.direction == 'left':
			self.rect.x = self.rect.x - self.speed
		elif self.direction == 'right':
			self.rect.x = self.rect.x + self.speed
		else:
			newdir = random.randrange (4)
			if newdir == 0:
				self.direction = 'up'
			elif newdir == 1:
				self.direction = 'down'
			elif newdir == 2:
				self.direction = 'left'
			elif newdir == 3:
				self.direction = 'right'
		stop = False

		for rock in rocks:
			if self.rect.colliderect (rock.rect):
				stop = True
				break
		if self.rect.left < 0:
			stop = True
		elif self.rect.top < 0:
			stop = True
		elif self.rect.right > SCREENWIDTH:
			stop = True
		elif self.rect.bottom > SCREENHEIGHT:
			stop = True
		
		if stop:
			self.rect.x = ox
			self.rect.y = oy
			self.direction = 'stop'

class Helium (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y

class Weapon (pygame.sprite.DirtySprite):
	def __init__(self, x, y, img, groups):
		pygame.sprite.Sprite.__init__(self) 
		self.image = img
		self.groups = groups
		self.image.set_colorkey(WHITE)
		self.rect = self.image.get_rect()
		self.rect.x = x
		self.rect.y = y

def game2 (gamer):
	redo = True
	rock_img = pygame.image.load('rock.png').convert()
	miner_img = pygame.image.load('miner.png').convert()
	badminer_img = pygame.image.load('badminer.png').convert()
	helium_img = pygame.image.load('helium.png').convert()
	weapon_img = pygame.image.load('weapon.png').convert()
	background_img = pygame.image.load('background_game2.png').convert()

	baddie_sprites = pygame.sprite.RenderUpdates ()
	player_sprites = pygame.sprite.RenderUpdates ()
	rock_sprites = pygame.sprite.RenderUpdates ()
	helium_sprites = pygame.sprite.RenderUpdates ()
	weapon_sprites = pygame.sprite.RenderUpdates ()
	all_sprites = pygame.sprite.RenderUpdates ()
	
	for x in xrange (SCREENWIDTH/25):
		for y in xrange (SCREENHEIGHT/25):
			rock = Rock (x * 25, y * 25, rock_img, [all_sprites, rock_sprites])
			rock_sprites.add (rock)
			all_sprites.add (rock)
	
	done = False
	first = True
	while not done:
		if redo == True:
			SCREEN.blit (background_img, (0, 0))
			pygame.display.update ()
			redo = False
			updates = []
			player = Miner (0, 100, miner_img, [all_sprites, player_sprites])
			player_sprites.add (player)		
			all_sprites.add (player)		
			
		while len (weapon_sprites) < 3:
			weapon = Weapon (random.randrange(SCREENWIDTH/25) * 25, random.randrange(SCREENHEIGHT/25) * 25, weapon_img, [all_sprites, baddie_sprites])
			weapon_sprites.add (weapon)
			all_sprites.add (weapon)
			pygame.sprite.groupcollide (weapon_sprites, rock_sprites, False, True)
			if not first:
				player.weapon = pygame.time.get_ticks() + 10000

		while len (baddie_sprites) < 6:
			baddie = BadMiner (random.randrange(SCREENWIDTH/25) * 25, random.randrange(SCREENHEIGHT/25) * 25, badminer_img, [all_sprites, baddie_sprites])
			baddie_sprites.add (baddie)
			all_sprites.add (baddie)
			pygame.sprite.groupcollide (baddie_sprites, rock_sprites, False, True)
			if not first:
				helium = Helium (random.randrange(SCREENWIDTH/25) * 25, random.randrange(SCREENHEIGHT/25) * 25, helium_img, [all_sprites, helium_sprites])
				helium_sprites.add (helium)
				all_sprites.add (helium)
				pygame.sprite.groupcollide (helium_sprites, rock_sprites, False, True)
				
		first = False
			
		baddie_sprites.update (rock_sprites)
		
		if len (player_sprites) == 0:
			redo = True
			gamer.lives = gamer.lives - 1
			if gamer.lives > 0:
				staticScreen ('died')
			
		if gamer.lives == 0:
			done = True
		if player.helium >= 5:
			done = True
		
		for event in pygame.event.get():
			if event.type == QUIT:
				sys.exit()	

		keys = pygame.key.get_pressed()  #checking pressed keys
		player.direction = 'stop'
		if keys[pygame.K_LEFT]:
			player.direction = 'left'
		if keys[pygame.K_RIGHT]:
			player.direction = 'right'
		if keys[pygame.K_UP]:
			player.direction = 'up'
		if keys[pygame.K_DOWN]:
			player.direction = 'down'
			
		player_sprites.update(helium_sprites, weapon_sprites)
		pygame.sprite.groupcollide (player_sprites, rock_sprites, False, True)
		pygame.sprite.groupcollide (player_sprites, weapon_sprites, False, True)
		if pygame.sprite.spritecollideany(player, helium_sprites) is not None:
			player.helium = player.helium + 1
		pygame.sprite.groupcollide (player_sprites, helium_sprites, False, True)
		meDie = True
		theyDie = False
		if player.weapon > pygame.time.get_ticks():
			meDie = False
			theyDie = True
		pygame.sprite.groupcollide (player_sprites, baddie_sprites, meDie,theyDie)
				
		all_sprites.clear (SCREEN, background_img)
		updates.extend (all_sprites.draw (SCREEN))
		
		updates.append (pygame.draw.rect (SCREEN, BLACK, Rect (0, 0, SCREENWIDTH, 15)))
		menuText ((0, 0), WHITE, BLACK, "Lives " + str (gamer.lives))
		menuText ((90, 0), WHITE, BLACK, "Score " + str (gamer.score))
		yn = 'No'
		if player.weapon > pygame.time.get_ticks():
			yn = 'Yes'
		menuText ((180, 0), WHITE, BLACK, "Weapon? " + yn)
		menuText ((300, 0), WHITE, BLACK, "Helium " + str (player.helium))
		updates.append (menuText ((0, 20), WHITE, BLACK, "Collect weapon (green) kill enemy (red) then collect helium (yellow)"))
		
		pygame.display.update (updates)
		updates = []
		FPSCLOCK.tick (FPS)	
		#print round (FPSCLOCK.get_fps())
	return gamer
	
def game3 (gamer):
	pass

def doGame ():
	gamer = Gamer ()
	if gamer.lives > 0:
		gamer = game1 (gamer)
	if gamer.lives > 0:
		gamer = game2 (gamer)
	if gamer.lives > 0:
		gamer = game3 (gamer)
		
	if gamer.lives > 0:
		staticScreen ('won')
	else:
		staticScreen ('lost')

titleScreen ()

#gamer = Gamer ()
#game2 (gamer)

pygame.quit()
sys.exit()

"""
Day 1:
Exciting - game theme chosen! Let's get started!
I decided to do a series of 'mini games' so I can complete each one and make as many more as time permits
Made an empty 'skeleton' game with placeholders for title screen, info screen, winning screen, losing screen, a death screen
Level 1 - land a lunar module on the moon
Spent longer on the random stars than I should have... especially as according to the NASA photos you can't see stars on the moon...
Todo:
Make proper backgrounds
Maybe sounds for button presses in menus
Make background stars varying sizes or at least smaller!

Day 2:
Got a mostly working first level for the game, trying to think of what to do with level 2
Came up with an idea for level 2... started it :) it's some sort of cross between dig dug and pacman
It's progressing nicely, but having a bit of a problem with fps due to excessive sprites
Todo:
Scoring in level one is wrong :p
Thinking I should *actually* draw some moon for the landing pad to sit on...
Make proper backgrounds
Make proper graphics
Sounds for in game events (but it's in space - you can't actually hear explosions!)
Maybe sounds for button presses in menus
Make background stars varying sizes or at least smaller!

Day 3:
Misses most of day 3 :(

Day 4:
Got level 2 into a working state... I think there is time for a level 3
Todo:
Scoring in level one is wrong :p
Thinking I should *actually* draw some moon for the landing pad to sit on...
Make proper backgrounds
Make proper graphics
Sounds for in game events (but it's in space - you can't actually hear explosions!)
Maybe sounds for button presses in menus
Make background stars varying sizes or at least smaller!
Level 2 doesn't have any scoring...
Graphics, backgrounds, sounds and sprites :p
Level 2 has terrible framerate at times :/
Level 2 player sometimes spawns on enemy - equally enemy may spwn on player, weapon or helium  may spawn on anything...

Day 5:

Day 6:

Day 7:

"""

"""
What have I used:
Archlinux
Geany
Python 2
Pygame
Gimp
"""
